@@ -17,13 +17,17 @@ comp_neg_emi = 'EMI'
 comp_neg_npr_out = 'NPR, 2018-07-01'
 comp_neg_death = 'DTH'
 comp_neg_interprov_out = 'IOM'
-comp_neg_intraprov_out = 'RAO'
+comp_neg_intraprov_out = None
+comp_neg_etc = ['RAO']
+comp_neg_put_etc_before = False
 comp_pos_temp_in = None
 comp_pos_ret_emi = 'RE'
 comp_pos_npr_in = 'NPR, 2019-07-01'
 comp_pos_immi = 'IMM'
 comp_pos_interprov_in = 'IIM'
-comp_pos_intraprov_in = 'RAI'
+comp_pos_intraprov_in = None
+comp_pos_etc = ['RAI']
+comp_pos_put_etc_before = False
 comp_end = [comp_pos_npr_in]
 method_all = ['1dist', 'filler', 'prop']
 method = '1dist'
@@ -36,84 +40,29 @@ pop_start = 'Initial Population'
 
 
 
-def is_int(x, opname):
-    if not isinstance(x, int):
-        type_x = type(x)
-        raise TypeError(f"{opname} must be an int, not of {type_x}.")
+def is_type(x, opname, supposed_type_x):
+    if not isinstance(x, supposed_type_x):
+        raise TypeError(
+            f"{opname} must be {supposed_type_x}, not of {type(x)}."
+        )
     return None
 
+def is_bool(x, opname):
+    return is_type(x, opname, bool)
+
+def is_int(x, opname):
+    return is_type(x, opname, int)
+
 def is_str(x, opname, can_be_None = True):
     if x is None:
         if can_be_None:
             return None
         else:
-            msg = "{0} cannot be None; it must be a str."
-            msg = msg.format(opname)
-            raise TypeError(msg)
-    if not isinstance(x, str):
-        msg = "{0} must be a str, not of {1}."
-        msg = msg.format(opname, type(x))
-        raise TypeError(msg)
-    return None
-
-def is_length_n(x, opname, n):
-    if len(x) != n:
-        msg = "{0} must be of length {1}, not {2}."\
-            .format(opname, n, len(x))
-        raise ValueError(msg)
-    return None
-
-def is_list_int_ordered(x):
-    if not isinstance(x, list):
-        msg1 = "age.interval must be a list, not of type {}."
-        raise TypeError(msg1.format(type(x)))
-    is_length_n(x, 'age.interval', 2)
-    if x != sorted(x):
-        msg3 = "age.interval must be ordered."
-        raise ValueError(msg3)
-    if not isinstance(x[0], int):
-        msg4_1 = "The lower bound must be of type int, not {}."
-        raise ValueError(msg4_1.format(type(x[0])))
-    if not isinstance(x[1], int):
-        msg4_2 = "The upper bound must be of type int, not {}."
-        raise ValueError(msg4_2.format(type(x[1])))
-    if x[0] > 0:
-        msg5_1 =\
-            "The lower bound of age.interval must be nonpositive, " +\
-            "but the current value is {}."
-        raise ValueError(msg5_1.format(x[0]))
-    if x[1] < 0:
-        msg5_2 =\
-            "The upper bound of age.interval must be nonnegative, " +\
-            "but the current value is {}."
-        raise ValueError(msg5_2.format(x[1]))
-    return None
-
-def is_list_str(x, opname, n = None):
-    if not isinstance(x, list):
-        msg1 =\
-            "{0} must be a list, not of {1}."\
-            .format(opname, type(x))
-        raise TypeError(msg1)
-    if x == []:
-        msg2 = "{0} must not be an empty list.".format(opname)
-        raise ValueError(msg2)
-    if n is not None:
-        is_length_n(x, opname, n)
-    for i, item in enumerate(x):
-        if not isinstance(item, str):
-            msg3 =\
-                "{0} has an item at index {1} that is not of type str " +\
-                "but of {2}."\
-                .format(opname, i, type(item))
-            raise ValueError(msg3)
-    return None
+            raise TypeError(f"{opname} cannot be None; it must be a str.")
+    return is_type(x, opname, str)
 
 def is_nonnegative(x, opname, force_positive = False):
-    if not isinstance(x, int):
-        raise TypeError(
-            f"{opname} must be an integer, not of {type(x)}."
-        )
+    is_type(x, opname, int)
     x_test = None
     if force_positive:
         x_test = (x <= 0)
@@ -125,27 +74,32 @@ def is_nonnegative(x, opname, force_positive = False):
         )
     return None
 
-def is_list_containing(x, opname):
-    if not isinstance(x, list):
-        msg1 =\
-            "{0} must be a list, not of {1}."\
-            .format(opname, type(x))
-        raise TypeError(msg1)
+def is_list_containing(x, opname, check_subset: bool = True):
+    is_type(x, opname, list)
     for i, item in enumerate(x):
         if not isinstance(item, str):
-            msg2 =\
-                "{0} has an item at index {1} that is not of type str " +\
-                "but of {2}."\
-                .format(opname, i, type(item))
-            raise ValueError(msg2)
-    comps = [
-        comp_neg_temp_out, comp_neg_emi, comp_neg_npr_out, 
-        comp_neg_death, comp_neg_interprov_out, comp_neg_intraprov_out,
-        comp_pos_temp_in, comp_pos_ret_emi, comp_pos_npr_in,
-        comp_pos_immi, comp_pos_interprov_in, comp_pos_intraprov_in
-    ]
-    msg3 = "{0} is not a subset of all the components.".format(x)
-    assert set(x).issubset(comps), msg3
+            raise ValueError(
+                f"{opname} has an item at index {i} that is not of "
+                f"type str but of {type(item)}."
+            )
+    if check_subset:
+        comps =\
+            [
+                comp_neg_temp_out, comp_neg_emi, comp_neg_npr_out, 
+                comp_neg_death, 
+                comp_neg_interprov_out, comp_neg_intraprov_out
+            ] +\
+                comp_neg_etc +\
+            [
+                comp_pos_temp_in, comp_pos_ret_emi, comp_pos_npr_in,
+                comp_pos_immi, 
+                comp_pos_interprov_in, comp_pos_intraprov_in
+            ] +\
+                comp_pos_etc
+        if not set(x).issubset(comps):
+            raise ValueError(
+                f"{x} is not a subset of all the components."
+            )
 
     return None
 
@@ -155,6 +109,11 @@ def is_method(x):
 
     return None
 
+is_containing_comp_end = lambda x: is_list_containing(x, 'comp.end')
+is_containing_netc = lambda x: is_list_containing(x, 'comp_neg.etc', False)
+is_containing_petc = lambda x: is_list_containing(x, 'comp_pos.etc', False)
+is_etc_n_b4_inters = lambda x: is_bool(x, 'comp_neg.put_etc_before')
+is_etc_p_b4_inters = lambda x: is_bool(x, 'comp_pos.put_etc_before')
 is_int_age_max = lambda x: is_int(x, 'age.max')
 is_int_age_down_to = lambda x: is_int(x, 'age.down_to')
 is_pos_age_prop_size = lambda x: is_nonnegative(x, 'age.prop_size', True)
@@ -176,7 +135,6 @@ is_str_pop_birth = lambda x: is_str(x, 'pop.birth', False)
 is_str_pop_end = lambda x: is_str(x, 'pop.end', False)
 is_str_pop_sex = lambda x: is_str(x, 'pop.sex', False)
 is_str_pop_start = lambda x: is_str(x, 'pop.start', False)
-is_list_containing_comp_end = lambda x: is_list_containing(x, 'comp.end')
 
 
 
@@ -195,7 +153,9 @@ _global_config = {
         'npr_out': comp_neg_npr_out,
         'death': comp_neg_death,
         'interprov_out': comp_neg_interprov_out,
-        'intraprov_out': comp_neg_intraprov_out
+        'intraprov_out': comp_neg_intraprov_out,
+        'etc': comp_neg_etc,
+        'put_etc_before': comp_neg_put_etc_before
     },
     'comp_pos': {
         'temp_in': comp_pos_temp_in,
@@ -203,7 +163,9 @@ _global_config = {
         'npr_in': comp_pos_npr_in,
         'immi': comp_pos_immi,
         'interprov_in': comp_pos_interprov_in,
-        'intraprov_in': comp_pos_intraprov_in
+        'intraprov_in': comp_pos_intraprov_in,
+        'etc': comp_pos_etc,
+        'put_etc_before': comp_pos_put_etc_before
     },
     'method': method,
     'pop': {
@@ -256,7 +218,7 @@ _registered_options = {
             '\n (list of str)\n '+\
             'A list of components that denote the end-of-period ' +\
             'figures. This must be a subset of all the components.',
-        validator = is_list_containing_comp_end
+        validator = is_containing_comp_end
     ),
     'comp_neg.temp_out': RegisteredOption(
         key = 'comp_neg.temp_out',
@@ -312,6 +274,35 @@ _registered_options = {
             "during the period in a population table.",
         validator = is_str_comp_neg_intraprov_out
     ),
+    'comp_neg.etc': RegisteredOption(
+        key = 'comp_neg.etc',
+        defval = comp_neg_etc,
+        doc =\
+            "\n (list of str)\n " +\
+            "A list of negative components that is not a part of " +\
+            "conventional components. Components in this list is " +\
+            "considered after all the conventional negative components " +\
+            "have been dealt with. If no extra negative components " +\
+            "exist in the table, let this be `[]`, an empty list.",
+        validator = is_containing_netc
+    ),
+    'comp_neg.put_etc_before': RegisteredOption(
+        key = 'comp_neg.put_etc_before',
+        defval = comp_neg_put_etc_before,
+        doc =\
+            "\n (bool)\n " +\
+            "A bool (`False` by default) that determines whether " +\
+            "or not components in `comp_neg.etc` should precede " +\
+            "`comp_neg.interprov` and `comp_neg.intraprov` when " +\
+            "applying corrections. If `True`, the precedence of " +\
+            "components in `comp_neg.etc` will be between " +\
+            "conventional negative components and interprovincial " +\
+            "migrations. If `False`, it is " +\
+            "set to follow interprovincial or intraprovincial " +\
+            "migration, depending on the regional level of the " +\
+            "population tale.",
+        validator = is_etc_n_b4_inters
+    ),
     'comp_pos.temp_in': RegisteredOption(
         key = 'comp_pos.temp_in',
         defval = comp_pos_temp_in,
@@ -366,6 +357,35 @@ _registered_options = {
             "during the period in a population table.",
         validator = is_str_comp_pos_intraprov_in
     ),
+    'comp_pos.etc': RegisteredOption(
+        key = 'comp_pos.etc',
+        defval = comp_pos_etc,
+        doc =\
+            "\n (list)\n " +\
+            "A list of positive components that is not a part of " +\
+            "conventional components. Components in this list is " +\
+            "considered after all the conventional positive components " +\
+            "have been dealt with. If no extra positive components " +\
+            "exist in the table, let this be `[]`, an empty list.",
+        validator = is_containing_petc
+    ),
+    'comp_pos.put_etc_before': RegisteredOption(
+        key = 'comp_pos.put_etc_before',
+        defval = comp_pos_put_etc_before,
+        doc =\
+            "\n (bool)\n " +\
+            "A bool (`False` by default) that determines whether " +\
+            "or not components in `comp_pos.etc` should precede " +\
+            "`comp_pos.interprov` and `comp_pos.intraprov` when " +\
+            "applying corrections. If `True`, the precedence of " +\
+            "components in `comp_pos.etc` will be between " +\
+            "conventional positive components and interprovincial " +\
+            "migrations. If `False`, it is " +\
+            "set to follow interprovincial or intraprovincial " +\
+            "migration, depending on the regional level of the " +\
+            "population tale.",
+        validator = is_etc_p_b4_inters
+    ),
     'method': RegisteredOption(
         key = "method",
         defval = method,
